--------------------------------------------------------------------------------------------------------------------------------------------------------
--This file is the center of the system and initiates the interactions with other scripts
--------------------------------------------------------------------------------------------------------------------------------------------------------

local M = {}

local celestialsHandler  = require("scenario/gravitationalRacing/celestial/celestialsHandler")
local checkpointsHandler = require("scenario/gravitationalRacing/scenario/track/checkpointsHandler")
local shortcutHandler    = require("scenario/gravitationalRacing/scenario/track/shortcutHandler")
local ClassVehicle       = require("scenario/gravitationalRacing/classes/classVehicle")

local vehicles = {}

local sourceFile = ""

---------
-- local streams = require("vehicle/guistreams")

-- local function getRollPitch(dirFront, dirUp)
--   -- find vehicle roll and pitch, in degrees, 0deg being normal upright rotation, +/-180deg being on its roof
--   local dirLeft = dirUp:cross(dirFront)
--   local roll  = math.deg(math.asin(dirLeft.z))
--   local pitch = math.deg(math.asin(dirFront.z))
--   if dirUp.z < 0 then -- if we are closer to upside down than to downside up
--     -- detect the "on its roof" situation, where angles are zero, and make sure they go all the way to 180deg instead, like this:
--     -- original rotation angles:  0deg (ok), 90deg (halfway),      0deg (on its roof), -90deg (halfway), 0deg (ok)
--     -- corrected rotation angles: 0deg (ok), 90deg (halfway), +/-180deg (on its roof), -90deg (halfway), 0deg (ok)
--     roll  = sign( roll)*(180 - math.abs( roll))
--     pitch = sign(pitch)*(180 - math.abs(pitch))
--   end
--   --log("D", "recovery", "Roll: "..r(roll,2,2)..", Pitch: "..r(pitch,2,2)..", dirUp: "..s(recPoint.dirUp, 2,2))
--   return roll, pitch
-- end

-- local obj = scenetree.findObject("scenario_player0")
-- print(getRollPitch(vec3(obj:getDirectionVector()), vec3(obj:getDirectionVectorUp())))
---------

local function update(dt, start)
  if vehicles.scenario_player0 then
    celestialsHandler.update(start, vehicles, dt)
    checkpointsHandler.update(start, vehicles.scenario_player0, dt)
  end
end

local function setSourceFile(srcFile)
  sourceFile = srcFile
end

local function resetVehicle(vehicle)
  --[[
  Resets the vehicle, given the vehicle name
  ]]--
  --Note: this will always be false for a sceanrio outside of these mod scenarios
  if vehicles[vehicle] then
    vehicles[vehicle]:scheduleReset()
  end
end

local function resetVehiclePos(vehicle)
  --[[
  Resets a vehicle's internal pos
  This prevents the car from triggering anything while a scenario restart takes place
  ]]--
  if vehicles[vehicle] then
    vehicles[vehicle]:resetPosition()
  end
end

local function initialise(srcFile)
  local fullReset = sourceFile ~= srcFile

  if srcFile:find("hubworld") or srcFile:find("solar_system_simulation") then
    celestialsHandler.createCelestials(fullReset, true)
    celestialsHandler.printResults()
  else
    celestialsHandler.createCelestials(fullReset, false)
    celestialsHandler.printResults()

    --Setup checkpoints
    checkpointsHandler.createCheckpoints(fullReset, srcFile)
    checkpointsHandler.printResults()
    --Setup shortcuts
    shortcutHandler.initialise(fullReset, srcFile)
  end

  if not vehicles.scenario_player0 then
    vehicles.scenario_player0 = ClassVehicle.new("scenario_player0", scenetree.findObject("scenario_player0"):getPosition())
  else
    resetVehiclePos("scenario_player0")
  end

  sourceFile = srcFile
end

M.update = update
M.setSourceFile = setSourceFile
M.resetVehicle = resetVehicle
M.initialise = initialise
return M
