angular.module('beamng.apps')
.directive('scenarioEndScreen', ['bngApi', 'gamepadNav', function (bngApi, gamepadNav) {
  return {
    templateUrl: 'modules/apps/ScenarioEndScreen/templateBMNG.html',
    replace: true,
    link: function (scope, element, attrs) {
      'use strict';

      function resetValues () {
        scope.$evalAsync(function () {
          document.getElementById("info").style.display = "none";
        });
      }

      scope.$on('gravitationalRacingScenarioEndScreen', function (event, data) {
        // var crossfireEnabled = gamepadNav.crossfireEnabled();
        // var gamepadNavEnabled = gamepadNav.gamepadNavEnabled();
        // var spatialNavEnabled = gamepadNav.spatialNavEnabled();
        // gamepadNav.enableCrossfire(false);
        // gamepadNav.enableGamepadNav(false);
        // gamepadNav.enableSpatialNav(true);
        // Utils.waitForCefAndAngular(() => {
        // bngApi.engineLua("extensions.core_input_bindings.menuActive(true);");
        // });

        /*
        *Adds extra zeros onto the front of a number until it is n digits long
        */
        function format(value, numDigits) {
          var strVal = value.toString();
          var digits = strVal.length;

          if (digits < numDigits) {
            for (var i = digits; i < numDigits; i++) {
              strVal = "0" + strVal;
            }
          }

          return strVal;
        }

        /*
        * Loads a given scenario
        */
        function loadScenario(filePath) {
          filePath = bngApi.serializeToLua(filePath);
          var cmd = "scenario_scenariosLoader.start(scenario_scenariosLoader.loadScenario(" + filePath + ", nil, " + filePath + "))";
          bngApi.engineLua(cmd);
        }

        /*
        *Restarts the current scenario
        */
        function restartScenario() {
          bngApi.engineLua("scenario_scenarios.restartScenario()");
        }

        /*
        Shows the unlocks
        */
        function showUnlocks() {
          var unlocks = document.getElementById("unlocks");
          var buttons = document.getElementById("buttons");

          // style='animation:fadeIn 1s, shimmer 2s linear forwards'
          var div = "<div>";

          for (let i = 0; i < 3; i++) {
            if (unlocked[i]) {
              let scenario = unlocked[i].name;
              let difficulty = unlocked[i].difficulty;

              // var button = document.createElement("li");
              // button.id = scenario;
              // button.className = "button";
              // button.innerHTML = scenario;

              // buttons.appendChild(button);

              div += "<span style='color:" + difficultyToColour(difficulty.replace(/["'\(\)]/g, "")) + "'>NEW TRACK UNLOCKED: " + scenario.toUpperCase() + " (" + difficulty.toUpperCase() + ")</span><br>";
              //Needs to be let specifically so that filePath is a new scoped variable everytime
              // let filePath = "levels/smallgrid/scenarios/gravitationalRacing/tracks/" + scenario.replace(" ", "_").toLowerCase(); + ".json";
              // document.getElementById(scenario).addEventListener("click", function () {loadScenario(filePath)});
            } else {
              //Add empty line
              div += "<br>";
            }
          }

          div += "</div>";
          unlocks.innerHTML = div;

          buttons.innerHTML += "<li class='button' id='" + scenarioName + "' style='border:2px solid " + colour + "'>Retry</li>";
          document.getElementById(scenarioName).addEventListener("click", restartScenario, false);

          let filePath = "levels/smallgrid/scenarios/gravitationalRacing/hubworld.json";
          document.getElementById("hubWorld").addEventListener("click", function () {loadScenario(filePath)});

          shownUnlocks = true;
        }

        /*
        *Compares the best, set and goal times
        */
        function compare(tSet, tGoal, tBest, alreadyAchieved, elementID) {
          var anim = "";

          console.log(alreadyAchieved);

          if (tSet <= tGoal) {
            document.getElementById(elementID).style.color = "lightgreen";

            if (!alreadyAchieved) {
              var medalID = elementID.replace("Data", "Medal");
              var medalObj = document.getElementById(medalID);
              medalObj.style.opacity = "1";
              medalObj.style.animation = "fadeScale 2.5s, shimmer 2s infinite forwards";
            }

            anim = "shimmerGreenPurple";
          } else {
            document.getElementById(elementID).style.color = "red";
            anim = "shimmerRedPurple";
          }

          //If tBest is not set (ie. new save fie / scenario not tried yet), the values could be 0 (time) or -1 (resets)
          if (tSet < tBest || (elementID.search("time") === 0 && tBest === 0) || (elementID.search("resets") === 0 && tBest === -1)) {
            //Display a purple shimmer effects
            var text = document.getElementById(elementID);
            text.style.color = "purple";
            text.style.animation = anim + " 2s infinite forwards";
            document.getElementById(elementID.replace("Data", "Title")).innerHTML += "<span class='pb'>(PB)</span>";
          }

          if (!shownUnlocks) {showUnlocks();}
        }

        /*
        *toggles the UI from view
        */
        function toggleUI() {
          var info = document.getElementById("info");
          var button = document.getElementById("toggle");

          if (info.style.visibility === "visible") {
            info.style.visibility = "hidden";

            button.innerHTML = "&nbsp;show&nbsp;";
            button.className = "show";
          } else {
            info.style.visibility = "visible";

            button.innerHTML = "&nbsp;hide&nbsp;";
            button.className = "hide";
          }
        }

        /*
        *Animates the data graphic to show the value gotten and the target
        *If the value achieved is better than the target, it turns the value green,
        *else turns the value red (to indicate not achieved)
        *@param targetValueFormatted - optional (only applies to times)
        */
        function animateGraphic(achievedTime, targetValue, currentBest, type, elementID, targetValueFormatted){
          //The total time it should take to animate the timer
          const REAL_TIME = 3;

          var incrementMultiplier = 0;
          var t = 0;
          var currentValue = 0;
          var pendEnd = false;

          var timer = setInterval(function() {
            //Since this function updates every 10ms;
            t += 1/100;

            //f(t) = (t/K - 1)³ + 1
            incrementMultiplier = Math.pow((t/REAL_TIME - 1), 3) + 1;

            //Account for the x³ approaching 1 unlikely to ever be 1 exactly
            if (Math.abs(incrementMultiplier-1) <= Math.pow(10, -6)) {
              incrementMultiplier = 1;
              pendEnd = true;
            }

            currentValue = achievedTime*incrementMultiplier;

            if (type === "time") {
              //Format for time
              var milliseconds = format(Math.floor(currentValue) % 1000, 3);
              var seconds      = format(Math.floor((currentValue / 1000) % 60), 2);
              var minutes      = Math.floor(currentValue / 1000 / 60).toString();

              document.getElementById(elementID).innerHTML = minutes + ":" + seconds + "." + milliseconds + "/" + targetValueFormatted;
            } else if (type === "resets") {
              document.getElementById(elementID).innerHTML = parseInt(currentValue) + "/" + targetValue;
            }

            if (pendEnd) {
              clearInterval(timer);
              compare(currentValue, targetValue, currentBest, achievedMedalsPrev[type], elementID);
            }
          }, 10);
        }

        function displayResults(){
          bngApi.engineLua('Engine.Audio.playOnce("AudioGui", "event:>UI>Scenario End Counting")')

          animateGraphic(totalTime, timeToBeat, bestTime, "time", "timeData", timeToBeatFormatted);
          animateGraphic(resetsUsed, resetsToBeat, bestResets, "resets", "resetsData");

          document.getElementById("info").removeEventListener("animationend", displayResults);
          document.getElementById("toggle").addEventListener("click", toggleUI, false);
        }

        function start() {
          //Add in the scenario specific details to the results div
          document.getElementById("scenarioName").innerHTML = "<b>" + scenarioName + " <span style='color:" + colour + ";'>(" + difficulty + ")</span></b>";
          document.getElementById("resultsHeading").style["background-color"] = colour;
          document.getElementById("unlocksHeading").style["background-color"] = colour;

          //Add in speciic data to beat
          document.getElementById("timeData").innerHTML += timeToBeatFormatted;
          document.getElementById("resetsData").innerHTML += resetsToBeat;

          //Show already achieved medals
          document.getElementById("timeMedal")  .style.opacity = achievedMedalsPrev.time   ? 1 : 0;
          document.getElementById("resetsMedal").style.opacity = achievedMedalsPrev.resets ? 1 : 0;

          //Animate finish div
          var finish = document.getElementById("finish");
          finish.style.display = "block";
          finish.style.animation = "slideDownOut 3s linear forwards";

          //Animate results div after the finish div
          var info = document.getElementById("info");
          info.style.display = "block";
          info.style.animation = "slideDownPause 1.75s linear 2.5s forwards";
          //Add event listener to call the function for animating the results after the animation has finished
          info.addEventListener("animationend", displayResults, false);

          //Animate the button to do with the results div
          var button = document.getElementById("toggle");
          button.style.display = "block";
          button.style.animation = "slideDownPause 1.75s linear 2.5s forwards";
        }

        /*
        Returns the colour of the difficulty level
        */
        function difficultyToColour(difficulty) {
          difficulty = difficulty.toUpperCase()
          if (difficulty === "TUTORIAL") {
            return "white";
          } else if (difficulty === "BASIC") {
            return "lightgreen";
          } else if (difficulty === "ADVANCED") {
            return "gold";
          } else if (difficulty === "EXPERT") {
            return "red";
          } else if (difficulty === "INSANE") {
            return "dodgerblue";
          }
        }

        //Scenario information
        var scenarioName = data.scenarioName;
        var difficulty = data.difficulty.type;
        var colour = data.difficulty.colour;

        var unlocked = data.unlocks;
        var shownUnlocks = false;

        var achievedMedalsPrev = data.medalsGottenAlready;

        //Various times
        var totalTime           = data.times.set;
        var bestTime            = data.times.best;
        var timeToBeat          = data.times.target;
        var timeToBeatFormatted = data.times.targetFormatted;

        //Various resets
        var resetsUsed   = data.resets.set;
        var bestResets   = data.resets.best;
        var resetsToBeat = data.resets.target;

        start();
      });

      scope.$on('ScenarioResetTimer', resetValues);
    }
  };
}]);
